import pkg_resources, os, configparser
import ldmud

def python_efun_help(efunname: str) -> str:
    """
    SYNOPSIS
            string python_efun_help(string efunname)

    DESCRIPTION
            Returns the docstring for the given Python efun, if there is any.

    SEE ALSO
            python_reload(E)
    """
    config = configparser.ConfigParser()
    config['efuns'] = {}
    config.read(os.path.expanduser('~/.ldmud-efuns'))
    efunconfig = config['efuns']

    if not efunconfig.getboolean(efunname, True):
        return None

    ws = pkg_resources.WorkingSet()
    for entry_point in ws.iter_entry_points('ldmud_efun', efunname):
        doc = getattr(entry_point.load(), '__doc__', None)
        if doc:
            # Trim algorithm from PEP 257
            lines = doc.expandtabs().splitlines()
            indent = len(doc)

            #Determine indentation
            for line in lines[1:]:
                stripped = line.lstrip()
                if stripped:
                    indent = min(indent, len(line) - len(stripped))

            # Remove indentation
            trimmed = [lines[0].strip()]
            if indent < len(doc):
                for line in lines[1:]:
                    trimmed.append(line[indent:].rstrip())

            # Strip off trailing and leading blank lines:
            while trimmed and not trimmed[-1]:
                trimmed.pop()
            while trimmed and not trimmed[0]:
                trimmed.pop(0)

            # Return a single string:
            return '\n'.join(trimmed) + '\n'

def register():
    ldmud.register_efun("python_efun_help", python_efun_help)



def startup():
    """ Loads all registered packages that offer the ldmud_efun entry point.

    In the configuration file ~/.ldmud-efuns single efuns can be deactivated
    with entries like:

        [efuns]
        name_of_the_efun = off
    """

    import pkg_resources, traceback, sys, os, configparser
    import ldmud

    config = configparser.ConfigParser()
    config['efuns'] = {}
    config.read(os.path.expanduser('~/.ldmud-efuns'))
    efunconfig = config['efuns']

    for entry_point in pkg_resources.iter_entry_points('ldmud_efun'):
        if efunconfig.getboolean(entry_point.name, True):
            try:
                print("Registering Python efun", entry_point.name)
                ldmud.register_efun(entry_point.name, entry_point.load())
            except:
                traceback.print_exc()



import importlib, pkg_resources, site, sys, os, configparser
import ldmud

def reload_modules():
    """
    SYNOPSIS
            void python_reload()

    DESCRIPTION
            Reloads all Python efuns. All packages providing the "ldmud_efuns"
            entry point are loaded. If they were already loaded, they are reloaded.
            Then the entry point is executed to register the efuns.

            Before reloading the function on_reload() is called in the module.

    SEE ALSO
            python_efun_help(E)
    """
    importlib.reload(site)
    ws = pkg_resources.WorkingSet()
    modules = dict(sys.modules)
    reloaded = set()

    config = configparser.ConfigParser()
    config['efuns'] = {}
    config.read(os.path.expanduser('~/.ldmud-efuns'))
    efunconfig = config['efuns']

    for entry_point in ws.iter_entry_points('ldmud_efun'):
        if efunconfig.getboolean(entry_point.name, True):
            # Remove the corresponding modules from sys.modules
            # so they will be reloaded.
            names = entry_point.module_name.split('.')
            for module in ('.'.join(names[:pos]) for pos in range(len(names), 0, -1)):
                if not module in modules or module in reloaded:
                    break

                try:
                    sys.modules[module].on_reload()
                except:
                    pass

                del sys.modules[module]
                reloaded.add(module)
                print("Reload module", module)

            ldmud.register_efun(entry_point.name, entry_point.load())

def register():
    ldmud.register_efun("python_reload", reload_modules)



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="ldmud-efuns",
    version="0.1.0",
    author="LDMud Team",
    author_email="ldmud-dev@UNItopia.DE",
    description="Python Efun management package for LDMud",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/ldmud/python-efuns",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: ISC License (ISCL)",
        "Operating System :: OS Independent",
    ],
    entry_points={
        'ldmud_efun': [
              'python_reload    = ldmudefuns.reload:reload_modules',
              'python_efun_help = ldmudefuns.help:python_efun_help',
        ]
    },
    zip_safe=False,
)
